;
;  C Startup.asm - Control Cross-C all-purpose startup code.
;
;  Copyright (c) 2002 Softools, Inc.  All rights reserved.
;
;  Since this file is self-configuring using several assemble-time
;  prompts, it should be assembled separately for each application
;  program being developed.  Labels can be defined to bypass the
;  prompts - the WinIDE New Project Wizard in fact does this.
;
;  If a pre-startup module is used to initialize other hardware,
;  interrupt vectors, etc., the global entry point CSTART should
;  be jumped to from that module.  Otherwise, this module is placed
;  at location 0 and will handle the startup.  The state of interrupts
;  is NOT changed by this module.  Therefore, if a pre-startup module
;  turns them off, main () or this module should turn them back on.
;  The SP and a MMU and required Rabbit peripherals are initialized
;  by this module.
;
;  The following memory configuration is used in the Control Cross-C
;  environment.  Note that not all segments are necessarily used in
;  all applications.
;
;  Segment	Contains
;  ==================================================================
;  CODE 	Startup code, libraries and near compiled code.
;  STARTUP	Startup vector table used to dynamically run startup
;		  initialization code.
;  CONST	String literals, const objects and
;		  initializers for automatic aggregates.
;  DATA 	Initialized data area.	May be copied from ROM to RAM.
;  FARCODE	Code compiled as far functions in C.
;  FARDATA	Initialized far data area.  This is stored out of the
;		  logical address space.
;  BSS		Uninitialized data area.  This may be zeroed on startup.
;  FARBSS	Uninitialized far data area.  The startup code DOES NOT
;		  zero this space!
;  BSSZ 	Initialized by 0 data.	This may be zeroed on startup.
;  STACK	The amount reserved for the CPU stack
;		The remmaining area is left over and used for the dynamic heap.
;
;  --------------------------------------------------------------------------
;  A program is linked with SLINK using the following commands:
;  This applies to the command line tools only - the WinIDE has
;  a dialog that configures these settings.  The New Project Wizard
;  will create projects and these can be used as a starting point.
;    (This is only an example.	The addresses used need to be changed
;    to reflect the target system and application program.)
;
;  In the Softools WinIDE, the names in parens (e.g. (CODE)) are segment
;  definitions and take the same addresses (e.g. start=8000 sets logical
;  address to 8000).  Modules are included around segment definitions.
;
;  (CODE, start=0)		; Place CODE at 0
;  (CONST)			; Place constants next
;  (STARTUP)			; Special startup vector table
;  (DATA)			; Place DATA next in ROM
;  (FARCODE,overlap)		; Place far C functions next
; *(FARDATA)			; Initializers (should not be in logical space)
;  (IDATA<DATA,start=8000)	; Reserve space for copied initialized DATA
;  (BSS)			; BSS is after DATA (at 8000)
;  (BSSZ)			; BSSZ is after BSS
;  (STACK)			; The CPU stack follows data
; *(FARBSS,start=0,at=x,overlap); FARBSS can be anywhere in physical space
;  program modules and libraries should be listed here
;
; * Used only if 'far' is used in C.
;
;  --------------------------------------------------------------------------
;  The following assembly-time questions are asked to configure the
;    application:
;
;  We prompt for the CPU stack size.  This eliminates editing
;    this file just to change the stack size.
;
;  We prompt to find out if the system is battery backed.
;    If it is, then we do not zero-fill BSS and
;    We prompt if the system needs a power-up test for initialization.
;      If it does, then a variable stored in segment DATA is compared
;      to a known default value.  If the variable contains this value,
;      the initialization is skipped.  Otherwise, it is performed.
;    If not, segment BSS is never zero filled.
;
;  In order to eliminate the prompts below, which may be necessary
;    in the Windows WinIDEbug environment because a build stops for
;    each prompt, the startup questions can be supplid as EQUs.
;
;  If STACK_SIZE is supplied, BATT_BACKED, and BATT_TEST
;    can be supplied to override the questions.  If any but STACK_SIZE
;    is omitted, all others default to N.
;
Y	equ	1
N	equ	0
;
	.ifdef	STACK_SIZE
;
size	.equ	STACK_SIZE	; Application stack size?

	.ifdef	BATT_BACKED
bb_sys	.equ	Y
	.printx Building RAM battery backed C startup
	.else
bb_sys	.equ	N		; Battery backed system?
	.printx Building RAM initializing C startup
	.endif

	.ifdef	BATT_TEST
bb_test .equ	Y
	.printx Building validating RAM battery backed C startup
	.else
bb_test .equ	N	; Do battery backup test? (Don't care if bb_sys is N)
	.endif

	.ifndef	INC_DEBUG
INC_DEBUG .equ	Y
	.endif
	.if	! INC_DEBUG
	.printx Building NO DEBUG region C startup (cannot debug in WinIDEbug)
	.endif

	.else
;
size::	.input	"Application CPU stack size? "
bb_sys: .input	"Is application running in a battery-backed system (Y/N)? "
	.if	bb_sys
bb_test:.input	"Do data initialization if a battery-backed test fails (Y/N)? "
	.else
bb_test equ	N
	.endif
INC_DEBUG:.input "Include 2k debug kernel region (Y/N)? "
	.endif
;
	.include Rabbit2000.inc
;
__stack_size::	.equ	even size
;
	.cseg				; Define implicit segment ordering
	.defseg CONST
	.defseg STARTUP
	.dseg
	.defseg	FARCODE
	.bseg
	.defseg BSSZ
	.defseg IDATA,data
	.defseg STACK,stack,align=2	; Define STACK segment
	.defseg FARBSS,data
	.seg	STACK
_stackMarker::
	.ds	4			; A 4-byte marker that can check for stack overruns
	.ds	__stack_size
__stack::.equ	$			; Initial SP
;
	.cseg
	.phase	0
m	ljp	cstart			; This is at 0
inFlash:.equ	$-1			; Flash marker byte 3 (must not be 0!)
;
	.org	0Ah
	.ds	1			; Bootstrap leaves 19200 TC here
_GCDR:	.ds	1			; And GCDR here.
_TAT4R:	.ds	1			; And TAT4R here.
;
;  Debug output and input vectors.  The __stdio FILE pointer
;  is set to these entry points.  In production code, printf
;  calls these per character doing nothing.  In debug mode,
;  the debug kernel redirects these to output to the WinIDE.
;
	.org	10h			; Debug in vector
	ret
;
	.org	18h			; Debug out vector
	ret
;
;  Leave a RET at 50h in case a RST 28 is executed.  If setInterruptHandler
;  is called, this vector is copied when IIR is changed still preserving
;  the RET.  The application may still use setInterrupthandler to provide
;  a RST 28 handler so it can handle RST 28s.
;
	.org	50h
	ret
;
;  If INC_DEBUG is defined, cstart is built without a 2k hole normally
;  required for the debug kernel.  If you disable this, you cannot run
;  this in the debugger (even if the debug kernel is loaded).
;
	.if	INC_DEBUG
;
	.org	800h			; 800h is the base above the debug kernel
;
	.else
;
	.org	60h+40			; Leave hole for system ID block
;
	.endif
	.dephase
;
;  idBlock definition, defined as an address
;
_idBlock::	.equ	60h
;
	.seg	FARCODE
cstart::ljp	..0
..0:	ipset	3			; Being safe
	.debug	{ cstart -1 204 cstart end
	ld	sp, __stack
;
;  Initialize the MMU SEGSIZE and STACKSEG to map logical data areas to their
;  physical addresses.
;
        ld	a,(sgst BSS >> 8) & 0F0h
	ioi ld	(SEGSIZE),a
	ld	a,(sgat BSS >> 12) - (sgst BSS >> 12)
	ioi ld	(STACKSEG),a
;
	.if	INC_DEBUG
;
;  Test inFlash byte - will be 0 if the code is in RAM
;
	ld	a,(inFlash)		; Test for flash, skip init if not
	or	a			; Is non-0 in flash
	jr	z,..inRam
;
	.endif
;
;  Initialize Rabbit peripherals here.  This uses a table to
;  quickly load address and data to intialize the Rabbit processor.
;  Simply extend the table as needed.  The Watchdog is set to 2
;  seconds - plenty of time to get to main from here.
;
	ld	bc,..iTblSize		; Size
	ld	hl,..iTbl		; Table
..lp:   ld	e,(hl)			; Addr
	inc	hl
	ioi ldi
	jp	v,..lp
	jr      ..cstart
;
;  This defaults to having the Flash at 00000 and Flash at 80000.
;  Note: These do not update the shadow RAM locations.  If you need
;  to readback any registers used here, eiher initialize the RAM
;  (e.g. _srMMIDR is the MMIDR shadow location) or use the
;  shadow I/O functions in C to reinitialize them.
;
;  Note: These set chip selects to 0 wait states.  Do not enable
;  wait states - these tools do not support wait states.
;
..iTbl:
;
;  Set STACKSEG to 80000 - start of DATASEG
;  If RAM begins at C0000 change the 80 to C0.
;
	.db	STACKSEG,80h - (sgst BSS >> 12)
	.db	GCSR,	8	; GCSR: no periodic int, main osc no div
	.db	GCDR,	0
	.db	MMIDR,	0	; MMIDR: normal operation
	.db	MB0CR,	0C8h	; MB0CR: 0 ws, OE0, CS0 active, WP enabled
	.db	MB1CR,	0C2h	; MB1CR: 0 ws, OE0, WE0, CS2 active
	.db	MB2CR,	0C5h	; MB2CR: 0 ws, OE1, WE1, CS1 active
	.db	MB3CR,	0C5h	; MB3CR: 0 ws, OE1, WE1, CS1 active
	.db	TACSR,  001h	; TACSR: Enable timer A
	.db     WDTCR,	5Ah	; WDTCR: 2 Seconds to get to main
;
..iTblSize=($-..iTbl)/2
;
;  This table of IO addresses will be zeroed.  This is to ensure
;  that uninitialized Rabbit peripherals and registers come up
;  consistently in Flash and RAM execution.
;
..zTbl:	.db	0x64, 0x65, 0x66, 0x67, 0x68, 0x69, 0x6A
	.db	0x6B, 0x6C, 0x6D, 0x6E, 0x6F, 0x74, 0x77
	.db	0x78, 0x79, 0x7A, 0x7B, 0x7C, 0x7D, 0x7E
	.db	0x7F, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85
	.db	0x86, 0x87, 0x98, 0x99, 0xA2, 0xA3, 0xAB
	.db	0xAD, 0xAF, 0xB0, 0xB1, 0xB2, 0xB3, 0xB4
	.db	0xB5, 0xD4, 0xE4, 0xF4
;
..zTblSize=$-..zTbl
;
	.if	INC_DEBUG
;
;  Running in RAM at this point.  The folowing code can
;  be executed but only in RAM debugging - be careful
;  what is placed here - it won't run in the Flashed version.
;
..inRam:ipset	2			; So that serial break-outs work
	xor	a,a			; A NOP
	ld	(10h),a			; In RAM, remove RET to allow printfs
	ld	(18h),a			; Remove this to stub printf in RAM
	.endif
;
;  Zero Rabbit registers in zTbl
;
..cstart:
	xor	a,a
	ld	b,..zTblSize
	ld	hl,..zTbl
..zlp:	ld	e,(hl)
	ioi ld	(de),a
	inc	hl
	djnz	..zlp
;
;  If checking a battery backed system, this code defines in segment
;  DATA a test pattern and checks to see if it valid.  If it is not,
;  then BSS, BSSZ, and DATA are initialized.  If it matches, this
;  bypasses all initialization and continues below.
;
	.if	bb_test
	.dseg
bb_patt:db	0C3h,0F0h,0AAh,03Ch,0F0h,055h
	.seg	FARCODE
	ld	hl,bb_patt		; Test pattern
	ld	b,6			; 6-byte pattern
	ld	de,0
..test: ld	a,d
	add	(hl)
	ld	d,a
	ld	a,e
	xor	(hl)
	ld	e,a
	inc	hl
	djnz	..test			; All bytes XORed is 0
	ld	a,low (0C3h+0F0h+0AAh+03Ch+0F0h+055h)
	sub	d			; Check sum of data
	jr	nz,..fail
	or	e			; Data XORed is 0
	jr	z,..strt		; Skip initialization
..fail: .endif
;
;  Zero out the BSS data area.
;  This is NOT done if the system is battery-backed!
;
	.if	! bb_sys || bb_test
	ld	hl, sgst BSS		; Logical start of BSS
	ld	bc, sgsz BSS		; Size of BSS
..zero:	ld	a,b
	or	c
	jr	z,..bssz
	ld	(hl),0			; 0 first byte
	inc	hl
	dec	bc
	jr	..zero
	.endif
;
;  Zero out the BSSZ data area.  This is for data with initializers of
;  0 (e.g. int x = 0;) and MUST be done even in a battery backed system.
;  (Unless the battery-backed test passes and all initialization is
;  skipped.
;
..bssz: ld	hl, sgst BSSZ		; Start of BSSZ
	ld	bc, sgsz BSSZ		; Size of BSSZ
..loop: ld	a,b
	or	c
	jr	z,..data
	ld	(hl),0			; Zero BSSZ
	inc	hl
	dec	bc
	jr	..loop
;
;  Copy initialized data to segment DATA.
;
..data: ld	bc, sgsz DATA		; Size of DATA
	ld	a,b
	or	c
	jr	z, ..strt		; No data
	inc	b			; For separate dec's
	ld	hl, sgat DATA 		; Phys start of DATA
	ld	a, sgat DATA >> 16
	ld	ix, sgst IDATA 		; Logical start of IDATA
..copy:	ld	iy, hl
	ex	de, hl
	ldp	hl, (iy)
	ld	(ix), l			; Copy
	ex	de, hl
	inc	l			; Inc 20-bit pointer
	jr	nz,..c
	inc	h
	jr	nz,..c
	inc	a
..c:	inc	ix
	dec	c
	jr	nz,..copy
	djnz	..copy
;
;  Unconditionally zero all shadow registers regardless of battery back SRAM.
;
..strt:	ld	b,__endSR##-__startSR##	; Start and end markers
	ld	hl,__startSR##
..0sr:	ld	(hl),0
	inc	hl
	djnz	..0sr
;
;  Read GCSR and store reset code
;
	ioi ld	a,(GCSR)
	rlca
	rlca
	and	a,3
	ld	(_resetCode),a
	ld 	a,1
	ld	(_srTACSR##),a		; Set TACSR (timer A is running)
;
;  Initialize GCDR for no doubling and set shadow
;
	ld	a,(inFlash)		; Test for flash, skip init if not
	or	a			; Is non-0 in flash
	jr	nz,..ram
	ld	a,(_GCDR)		; Init srGCDR shadow
	ld	(_srGCDR##),a
	ld	a,(_TAT4R)
	ld	(_srTAT4R##),a
	ld	a,0C0h			; Init srPCSR
	ld	(_srPCFR##),a
;
;  Calculate 19200 TC using that to enable clock doubled mode if the
;  speed allows us to do so.  For low power operation, this can be
;  removed or reversed in main by writing a 0 to GCDR (using a shadow
;  out).  This sets up for a 20nS low time for < 14MHz xtals.
;  Modify the value if this doesn't suit your system.
;
..ram:	lcall   $SerialCalc19200TC
	ld	(hl),a			; Save TC
	cp	22			; 12.9MHz (doubling cutoff) / 2 / 16 / 19200 = 21
	jr	nc,..cont		; If > 21, do not double
	sla	(hl)			; Double divisor since speed will double
	ld	a,07h			; 20ns nominal low time, best for < 12MHz
	ld	(_srGCDR),a		; Set GCDR shadow
	ioi ld	(GCDR),a
..cont:
;
;  Get function pointers in segment STARTUP and call each one.
;  This allows compile-time installment of startup initialization
;  which can be executed before main.  The order of call is the
;  order of the addresses in the output.  In a single module they
;  are guaranteed to occur in order.  Modules linked ahead of others
;  are guaranteed to execute their STARTUP functions first.
;
	.ifdef	STARTUP_SEG
	ld	a,sgsz STARTUP / 2	; Number of entries
	or	a
	jr	z,..main
	ld	hl,sgst STARTUP
..glp:	ld	e,(hl)
	inc	hl
	ld	d,(hl)
	inc	hl
	push	hl
	push	af
	call	.callde
	pop	af
	pop	hl
	dec	a
	jr	nz,..glp
	.endif
;
;  Initialize __stdio to the vectors above.
;  Set the __stack_marker to known values to detect stack overruns.
;  Call main ().  Note, main must be far.  If it is declared near,
;  change the lcall to call and $main to _main.
;
..main:	ld	hl,__debugStdio
	ld	(__stdio),hl
	ld	ix,_stackMarker
	ld	hl,5555h
	ld	(ix),hl
	add	hl,hl			; AAAA
	ld	(ix+2),hl
	lcall	$main## 		; Just halt if it returns
	ld	a,(inFlash)
	or	a,a
	jr	nz,_abort		; If running in RAM
_mainExit::
	rst	28h			; break into debugger
;
;  Here, the following routines are stubbed, since there is little that
;  can be done in an embedded system.  If you DO want to handle these
;  somehow, remove the public entry points here.
;
_abort::
;
..halt: jr	..halt

.callde:
	push	de			; Calls address in DE
	ret
;
;  Calculate the timer constant for 19200.
;  Returned in A (also address of the TC byte in HL)
;
;  unsigned char  SerialCalc19200TC( void );
;
$SerialCalc19200TC::
	push	ip
	ipset	3		; Or interrupts interfere
	ld	de,1FFFh	; mask for RTC bits
..wait: ioi ld	(RTC0R),a	; load RTC registers
	ioi ld	hl,(RTC0R)	; get LSW RTC
	and	hl,de		; mask off bits
	jr	nz,..wait	; wait bits 0-9 = 0
	ld	a,5Ah
	ld	de,0000h	; DE = count
..loop: inc	de
	ld	b,49
	djnz	$
	ioi ld	(WDTCR),a
	ioi ld	(RTC0R),a
	ioi ld	hl,(RTC0R)	; get LSW RTC
	bit	4,h		; test bit 10
	jr    	z,..loop
	ld	a,d		; 19200 time constant
	ld	hl,_Serial19200TC
	pop	ip
	lret
end:	.debug	} end 513
;
;  Define variables defining free RAM available for the heap.
;  If the heap is moved, __heap_end must remain >= __heap_start.
;  If malloc/free are not used, then it doesn't matter.  These
;  must address memory that is always mapped in.  It's probable
;  but very important that the heap start is NOT 0.
;
;  We highly recommend against using malloc and free in an
;  embedded application.
;
__heap_start::
	dw	__stack 		; Stack top defines start of heap
__heap_end::
	dw	0DFFFh			; The end is at 0DFFF
;
	.bseg
;
;  Calculated time constant for TAT4R.  Use by serial libraries
;  to handle baud rates without regard to CPU speed.  If GCDR
;  is changed, this value must also be updated (going into
;  double clock mode means you must double this constant!)
;  The Softools serial library recalculates this value so there
;  is no need for the user to change this if GCDR is changed.
;
_Serial19200TC::
	.ds	1
;
;  Reason for reset is stored here:
;    00 - Program restarted at 0
;    01 - Watchdog caused reset
;    03 - Reset line caused reset
;
_resetCode::
	.ds	1
;
;  Define _stdio here so we don't bring in fassign just to use this variable
;
__stdio::
	.ds	2
;
;  An intialized FILE structure for intial printf redirection
;
	.dseg
__debugStdio:
	.dw	18h			; Output address
	.dw	10h			; Input address
	.dw	0			; Putback char
;
	.if	INC_DEBUG
start	=	cstart
	.else
start	=	0xFFFF			; FFFF means no start (stops autorun)
	.endif
	.end	start
